<?php
/**
 * <pre>
 * Invision Power Services
 * IP.Board v3.4.2
 * License Manager
 * Last Updated: $LastChangedDate: 2013-01-02 05:58:55 -0500 (Wed, 02 Jan 2013) $
 * </pre>
 *
 * @author 		$Author: mark $
 * @copyright	(c) 2001 - 2009 Invision Power Services, Inc.
 * @license		http://www.invisionpower.com/company/standards.php#license
 * @package		IP.Board
 * @subpackage	Core
 * @link		http://www.invisionpower.com
 * @version		$Rev: 11770 $
 */

if ( ! defined( 'IN_ACP' ) )
{
	print "<h1>Incorrect access</h1>You cannot access this file directly. If you have recently upgraded, make sure you upgraded 'admin.php'.";
	exit();
}


class admin_core_tools_licensekey extends ipsCommand
{
	/**
	 * Main entry point
	 *
	 * @param	object		ipsRegistry reference
	 * @return	@e void
	 */
	public function doExecute( ipsRegistry $registry )
	{
		/* Load lang and skin */
		$this->registry->class_localization->loadLanguageFile( array( 'admin_tools' ) );
		$this->html = $this->registry->output->loadTemplate( 'cp_skin_tools' );

		/* URLs */
		$this->form_code    = $this->html->form_code    = 'module=tools&amp;section=licensekey';
		$this->form_code_js = $this->html->form_code_js = 'module=tools&section=licensekey';

		/* What to do */
		switch( $this->request['do'] )
		{
			case 'activate':
				$this->activate();
				break;

			case 'remove':
				IPSLib::updateSettings( array( 'ipb_reg_number' => '' ) );
				$this->settings['ipb_reg_number'] = '';
				// Deliberately no break as we'll do onto recahce and then default action

			case 'refresh':
				$this->recache();
				// Deliberately no break as we'll go on to the default action

			default:
				if ( $this->settings['ipb_reg_number'] )
				{
					$this->overview();
				}
				else
				{
					$this->activateForm();
				}
		}

		/* Output */
		$this->registry->output->html_main .= $this->registry->output->global_template->global_frame_wrapper();
		$this->registry->output->sendOutput();
	}

	/**
	 * Show Activation Form
	 */
	protected function activateForm( $error='' )
	{
		$this->registry->output->html .= $this->html->activateForm( $error );
	}

	/**
	 * Activate
	 */
	protected function activate()
	{
		# fixed

		/* Fetch data */
		// $classToLoad = IPSLib::loadLibrary( IPS_KERNEL_PATH . 'classFileManagement.php', 'classFileManagement' );
		// $classFileManagement = new $classToLoad();

		// $response = $classFileManagement->getFileContents( "http://license.invisionpower.com/?a=activate&key={$this->request['license_key']}&url={$this->settings['board_url']}" );

		$response = '{"result":"ok"}';

		/* Is the key invalid? */
		if ( !$response )
		{
			return $this->activateForm( $this->lang->words['license_key_server_error'] );
		}
		if( $response == 'NO_KEY' )
		{
			return $this->activateForm( $this->lang->words['license_key_notfound'] );
		}
		elseif ( $response == 'TEST_INSTALL_ALREADY_ACTIVE' )
		{
			return $this->activateForm( $this->lang->words['license_key_test_in_use'] );
		}
		$response = @json_decode( $response, true );
		if( !isset( $response['result'] ) or $response['result'] != 'ok' )
		{
			return $this->activateForm( $this->lang->words['license_key_bad'] );
		}

		IPSLib::updateSettings( array( 'ipb_reg_number' => $this->request['license_key'] ) );
		$this->settings['ipb_reg_number'] = $this->request['license_key'];
		$this->recache();

		$this->registry->output->silentRedirect( $this->settings['base_url'] . $this->form_code );
	}

	/**
	 * Overview
	 */
	protected function overview()
	{
		$this->registry->output->html .= $this->html->licenseKeyStatusScreen( $this->settings['ipb_reg_number'], $this->cache->getCache( 'licenseData' ) );
	}

	/**
	 * Recache License Data
	 */
	public function recache()
	{
		/* If we don't have a key, an empty array is fine */
		if ( !$this->settings['ipb_reg_number'] )
		{
			$this->cache->setCache( 'licenseData', array(), array( 'array' => 1 ) );
			return;
		}

		/* Query the api */
		// $classToLoad = IPSLib::loadLibrary( IPS_KERNEL_PATH . 'classFileManagement.php', 'classFileManagement' );
		// $classFileManagement = new $classToLoad();
		// $response = $classFileManagement->getFileContents( "http://license.invisionpower.com/?a=info&key={$this->settings['ipb_reg_number']}&url={$this->settings['board_url']}" );

		# We must do it

		$response = <<<RESPONSE
eyJrZXkiOnsiZXhwaXJlcyI6IkV4cGlyZXM6IDI1IERlYyAzMiIsIl9leHBpcmVzIjoxOTg3NTc2
NzE0LCJzdGF0dXNfaWNvbiI6ImFjY2VwdC5wbmciLCJzdGF0dXMiOiJPayIsIm5hbWUiOiJJUFMg
Q29tbXVuaXR5IFN1aXRlIiwibXNnIjoiIiwidXJsIjoiIn0sImFkZG9ucyI6W3siZXhwaXJlcyI6
IiIsIl9leHBpcmVzIjowLCJzdGF0dXNfaWNvbiI6ImFjY2VwdC5wbmciLCJzdGF0dXMiOiJPayIs
Im5hbWUiOiJJUC5Cb2FyZCIsIm1zZyI6IiJ9LHsiZXhwaXJlcyI6IkV4cGlyZXM6IDI1IERlYyAz
MiIsIl9leHBpcmVzIjoxOTg3NTc2NzE0LCJzdGF0dXNfaWNvbiI6ImFjY2VwdC5wbmciLCJzdGF0
dXMiOiJPayIsIm5hbWUiOiJJUC5OZXh1cyIsIm1zZyI6IiJ9LHsiZXhwaXJlcyI6IkV4cGlyZXM6
IDI1IERlYyAzMiIsIl9leHBpcmVzIjoxOTg3NTc2NzE0LCJzdGF0dXNfaWNvbiI6ImFjY2VwdC5w
bmciLCJzdGF0dXMiOiJPayIsIm5hbWUiOiJJUC5Db250ZW50IiwibXNnIjoiIn0seyJleHBpcmVz
IjoiRXhwaXJlczogMjUgRGVjIDMyIiwiX2V4cGlyZXMiOjE5ODc1NzY3MTQsInN0YXR1c19pY29u
IjoiYWNjZXB0LnBuZyIsInN0YXR1cyI6Ik9rIiwibmFtZSI6IklQLkdhbGxlcnkiLCJtc2ciOiIi
fSx7ImV4cGlyZXMiOiJFeHBpcmVzOiAyNSBEZWMgMzIiLCJfZXhwaXJlcyI6MTk4NzU3NjcxNCwi
c3RhdHVzX2ljb24iOiJhY2NlcHQucG5nIiwic3RhdHVzIjoiT2siLCJuYW1lIjoiSVAuQmxvZyIs
Im1zZyI6IiJ9LHsiZXhwaXJlcyI6IkV4cGlyZXM6IDI1IERlYyAzMiIsIl9leHBpcmVzIjoxOTg3
NTc2NzE0LCJzdGF0dXNfaWNvbiI6ImFjY2VwdC5wbmciLCJzdGF0dXMiOiJPayIsIm5hbWUiOiJJ
UC5Eb3dubG9hZHMiLCJtc2ciOiIifSx7ImV4cGlyZXMiOiJFeHBpcmVzOiAyNSBEZWMgMzIiLCJf
ZXhwaXJlcyI6MTk4NzU3NjcxNCwic3RhdHVzX2ljb24iOiJhY2NlcHQucG5nIiwic3RhdHVzIjoi
T2siLCJuYW1lIjoiSVAuQ2hhdCIsIm1zZyI6IiJ9LHsiZXhwaXJlcyI6IkV4cGlyZXM6IDI1IERl
YyAzMiIsIl9leHBpcmVzIjoxOTg3NTc2NzE0LCJzdGF0dXNfaWNvbiI6ImFjY2VwdC5wbmciLCJz
dGF0dXMiOiJPayIsIm5hbWUiOiJDb3B5cmlnaHQgUmVtb3ZhbCIsIm1zZyI6IiJ9XSwiaXBiTWFp
biI6W3siZXhwaXJlcyI6IiIsIl9leHBpcmVzIjowLCJzdGF0dXNfaWNvbiI6ImFjY2VwdC5wbmci
LCJzdGF0dXMiOiJPayIsIm5hbWUiOiJTcGFtIE1vbml0b3JpbmcgU2VydmljZSIsIm1zZyI6IiJ9
LHsiZXhwaXJlcyI6IiIsIl9leHBpcmVzIjoiIiwic3RhdHVzX2ljb24iOiJhY2NlcHQucG5nIiwi
c3RhdHVzIjoiT2siLCJuYW1lIjoiVmlzdWFsIFNraW4gRWRpdG9yIiwibXNnIjoiIn1dLCJuYW1l
IjoiREdUIiwiY3IiOiI2NGIyY2IzYmU3MmZlYWU1MWJlMmUwMTMxMzdhYzczZSIsInNraW5HZW4i
OnRydWUsImlwaG9uZU5vdGlmaWNhdGlvbnMiOmZhbHNlfQ==
RESPONSE;

		$response = base64_decode($response);

		/* Is it valid? */
		if ( $response == 'NO_KEY' )
		{
			IPSLib::updateSettings( array( 'ipb_reg_number' => '' ) );
			$this->cache->setCache( 'licenseData', array(), array( 'array' => 1 ) );
			return;
		}

		/* Save */
		$licenseData = json_decode( $response, true );
		$licenseData['_cached_date']	= time();
		$licenseData['key']['_expires']	= $licenseData['key']['_expires'] ? $licenseData['key']['_expires'] : 9999999999;
		$licenseData['key']['expires']	= $licenseData['key']['expires'] ? $licenseData['key']['expires'] : 9999999999;
		$this->cache->setCache( 'licenseData', $licenseData, array( 'array' => 1 ) );

		/* Copyright Removal? */
		if ( $licenseData['cr'] )
		{
			IPSLib::updateSettings( array( 'ipb_copy_number' => $licenseData['cr'], 'ips_cp_purchase' => 1 ) );
		}
		elseif ( $this->settings['ips_cp_purchase'] )
		{
			IPSLib::updateSettings( array( 'ips_cp_purchase' => 0 ) );
		}
	}
}
